function [point,success]=p_correc(funcs,point,free_par,step_cnd,method,remesh_flag,...
    previous,varargin)
%% correct point using Newton iteration
% function [point,success]=p_correc(point0,free_par,step_cnd,method,adapt,
%                                 previous)
% INPUT:
%   funcs problem functions
%   point0 initial point guess
%   free_par free parameter numbers in N^d
%       step_cnd steplength condition(s) as point(s)
%   method method parameters
%   remeshflag if zero or absent, do not adapt mesh; if one, always adapt
%       previous (optional) previously computed branch point (used, in case
%            of periodic solutions or connecting orbits, to
%            minimize phase shift)
% OUTPUT:
%       point final solution
%       success nonzero for successfull computation
%
% (c) DDE-BIFTOOL v. 3.2a(310), 28/10/2018
%
% 
%
%%

%% optional parameters:
if nargin<=5
    remesh_flag=0;
end
if nargin<=6
    previous=[];
end
%% replace old stability and other info by empty fields if present:
point=dde_trim_point(feval(['dde_',point.kind,'_create'],point),point);
%% preprocess point, stepcond and functions
data=struct('funcs',funcs,'free_par',free_par,'method',method,...
    'step_cnd',step_cnd,'point',point,'previous',previous,'extracolumns',[]);
if isfield(method,'preprocess') && ~isempty(method.preprocess)
    data=feval(method.preprocess,data);
end
%% remesh if required
if isfield(data.method,'remesh') && data.method.remesh && ...
        remesh_flag>1 && mod(remesh_flag,data.method.adapt_mesh_before_correct)==0
    data.point=p_remesh(data.point);
    if ~isempty(data.previous)
        data.previous=p_remesh(data.previous,data.point);
    end
    for i=1:length(data.step_cnd)
        data.step_cnd(i)=p_remesh(data.step_cnd(i),data.point);
    end
end
%% perform Newton-Raphson iterations:
f=@(x)p_correc_rhs(data.funcs,data.method,data.point,data.free_par,...
    'x',x,'pref',data.previous,'step_cond',data.step_cnd,...
    'extrapar',data.extracolumns,varargin{:});
x0=dde_x_from_point(data.point,data.free_par);
[x,success]=dde_nsolve(f,x0,data.method);
data.point=dde_point_from_x(x,data.point,data.free_par);
point=data.point;
if isfield(method,'postprocess') && ~isempty(method.postprocess)
    point=feval(method.postprocess,point,data);
end
%% recorrect with adapted mesh if necessary
if success && isfield(method,'remesh') && method.remesh
    if isfield (point,'mesh') && ~isempty(point.mesh)
        ma=method.adapt_mesh_after_correct;
        if remesh_flag==1 || ( remesh_flag>1 && mod(remesh_flag,ma)==0 )
            % do not adapt mesh when p_nr=0
            % adapt mesh when p_nr=1
            % adapt mesh when p_nr>1 & mod(p_nr,ma)=0
            if success % adapt & correct again
                method2=method;
                method2.adapt_mesh_before_correct=1;
                method2.adapt_mesh_after_correct=0;
                [point,success]=p_correc(funcs,point,free_par,step_cnd,method2,2,previous);
            end
        end
    end
end

end
